@@ -17,15 +17,17 @@
 #   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 #
 package Ecila::tools::sqledit;
-use vars qw($head %default_templates);
+use vars qw(@ISA $head %default_templates);
 use strict;
 
 use Carp;
 use CGI;
+use Ecila::tools::mysql;
 use Ecila::tools::cgi;
-use Ecila::db;
 use Ecila::tools::tools;
 
+@ISA = qw(Ecila::tools::mysql);
+
 $head = "
 <body bgcolor=#ffffff>
 ";
@@ -186,18 +188,11 @@ _PAGES_
 
 #sub DESTROY {}
 
-sub new {
-    my($type) = @_;
-
-    my($self) = {};
-    bless($self, $type);
-    $self->initialize();
-    return $self;
-}
-
 sub initialize {
     my($self) = @_;
 
+    $self->Ecila::tools::mysql::initialize();
+
     my($config) = config_load("sqledit.conf");
     if(exists($config->{'functions'})) {
 	my($functions) = $config->{'functions'};
@@ -216,26 +211,8 @@ sub initialize {
     }
     %$self = (%$self, %$config) if(defined($config));
 
-    $config = config_load("install.conf");
-    %$self = (%$self, %$config) if(defined($config));
-    
-    $self->{'db'} = Ecila::db->new() if(!defined($self->{'db'}));
-    my($db) = $self->{'db'};
-    $db->resources_load('sqledit_schema', 'Ecila::tools::schema');
-    $db->connect_error_handler(sub { $self->connect_error_handler(@_); });
-
     $self->{'params'} = [ 'table', 'links_set', 'stack', 'context', 'conf', 'order', 'style', 'limit' ];
     $self->{'templates'} = { %default_templates };
-
-    $self->special_auth_initialize();
-}
-
-sub connect_error_handler {
-    my($self, $db_type, $error) = @_;
-
-    my($cgi) = $self->{'cgi'};
-    my($url) = $cgi->url('-absolute' => 1);
-    $self->serror("Could not connect to database because: <p><pre>%s</pre> Try <a href=$url?context=confedit&file=$db_type.conf>editing $db_type.conf</a> and check that the database server is running.", $error);
 }
 
 sub run {
@@ -279,18 +256,9 @@ sub selector {
     $cgi = Ecila::tools::cgi->new() if(!defined($cgi));
 
     my($verbose) = $cgi->param('verbose');
-    if(defined($verbose)) {
-	# these are 'skicky', only changing when param verbose is given
-	# verbose = 1 - minimal
-	# verbose = 2 - enables opt_error_stack
-	# verbose = 3 - spare
-	# verbose>= 4 - sets DBI->trace to verbose-3
-	$::opt_verbose = $verbose if defined $verbose;
-	$::opt_error_stack = ($::opt_verbose > 1);
-	DBI->trace($verbose-3) if defined $verbose;
-    }
+    $::opt_verbose = $verbose if(defined($verbose));
 
-    my($context) = '';
+    my($context);
     if(defined($cgi->param('context'))) {
 	$context = $cgi->param('context');
     } elsif($cgi->path_info()) {
@@ -300,19 +268,12 @@ sub selector {
 	$cgi->param('pathname', $pathname);
     }
     my($content) = $cgi->param('content') || "text/html";
-    #
-    # Unbuffered output so that gauge() shows
-    #
-    $| = 1;
     $cgi->nph(1) if(exists($self->{'nph'}) && $self->{'nph'} eq 'yes');
     print $cgi->header(-type => $content);
     if($context !~ /^[\w_]+$/io) {
-	print "'$context' is not a valid context name";
+	print "$context is not a valid context";
 	return "";
     }
-
-    return "" if(!$self->special_auth($cgi));
-
     if(exists($self->{'context_allow'}) &&
        ( $self->{'context_allow'} eq '*' ||
 	 $self->{'context_allow'} !~ /\b$context\b/)) {
@@ -321,10 +282,8 @@ sub selector {
     }
     my($html);
     my($error);
-    eval {
-	local($SIG{__DIE__});
-	$html = $self->${context}($cgi);
-    };
+    eval "\$html = \$self->${context}(\$cgi)";
+#    $html = $self->${context}($cgi);
     if($@) {
 	$error = $@;
 	if($error !~ /HTMLIZED/) {
@@ -353,57 +312,6 @@ sub selector {
     return $html;
 }
 
-#
-# Undocumented feature specific to CFCE may be usefull for others but not sure
-#
-sub special_auth {
-    my($self, $cgi) = @_;
-
-    if(exists($self->{'special_auth'})) {
-	my($spec) = $self->{'special_auth'};
-	my($password) = $cgi->param($spec->{'param'});
-	if(!defined($password)) {
-	    print istring("special_auth: missing %s", $spec->{'param'});
-	    return 0;
-	}
-	my($today) = $self->db()->date(time());
-	my($row);
-	eval {
-	    $row = $self->db()->exec_select_one("select * from $spec->{'table'} where $spec->{'field_date'} = '$today' and $spec->{'field_password'} = '$password'");
-	};
-	if($@) {
-	    my($error) = $@;
-	    print istring("special_auth: cannot query table : %s", $error);
-	    return 0;
-	}
-	if(!defined($row)) {
-	    print istring("special_auth: permission denied");
-	    return 0;
-	}
-    }
-    return 1;
-}
-
-sub special_auth_initialize {
-    my($self) = @_;
-
-    if(exists($self->{'special_auth'})) {
-	my($param) = $self->{'special_auth'}->{'param'};
-	push(@{$self->{'params'}}, $param);
-    }
-}
-
-#
-# Output characters so that the Navigator shows activity during
-# a lengthy process. This prevents timeout and provides feedback 
-# to the user.
-#
-sub gauge {
-    my($self) = @_;
-
-    print " " if(exists($self->{'cgi'}));
-}
-
 #
 # Only works on RedHat-5.2
 #
@@ -454,8 +362,22 @@ sub search_entry_tags {
 sub requests_check {
     my($self) = @_;
 
-    if(!$self->db()->table_exists('sqledit_requests')) {
-	$self->db()->exec($self->db()->schema('sqledit_schema', 'sqledit_request'));
+    if(!$self->table_exists('sqledit_requests')) {
+	my($schema) = "
+create table sqledit_requests (
+  rowid int not null auto_increment,
+
+  rwhere varchar(255),
+  rtable char(64),
+  rlinks varchar(255),
+  rorder char(64),
+  rparams varchar(255),
+  label char(32),
+
+  unique sqledit_requests1 (rowid)
+)
+";
+	$self->exec($schema);
     }
 }
 
@@ -598,7 +520,7 @@ sub sinsert_1 {
     my($self, $table) = @_;
     my($cgi) = $self->{'cgi'};
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
     my($comment);
     my(%insert);
@@ -631,7 +553,7 @@ sub sinsert_1 {
 	}
     }
 
-    my($primary) = $self->db()->insert($table, %insert);
+    my($primary) = $self->insert($table, %insert);
     $insert{$info->{'_primary_'}} = $primary;
     
     return (\%insert, $primary);
@@ -650,7 +572,7 @@ sub set_handle_param {
     my($new_value);
     my($old_value);
     if(defined($row)) {
-	$self->db()->dict_link($desc, $table, $field);
+	$self->dict_link($desc, $table, $field);
 	my($values) = $desc->{'values'};
 	my(%value2key) = map { $values->{$_} => $_ } keys(%$values);
 
@@ -689,7 +611,7 @@ sub row2edit {
 sub row2edit_1 {
     my($self, $table, $row, $use_default) = @_;
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
     my($html);
     my($fields) = $info->{'_fields_'};
@@ -699,7 +621,7 @@ sub row2edit_1 {
 	my($desc) = $info->{$field};
 	my($type) = $desc->{'type'};
 	my($value) = defined($row) ? $row->{$field} : undef;
-	my($value_quoted) = defined($value) ? Ecila::tools::cgi::myescapeHTML($value) : '';
+	my($value_quoted) = defined($value) ? CGI::escapeHTML($value) : '';
 	if($type eq 'char') {
 	    my($size);
 	    if($desc->{'size'} < 2) {
@@ -728,10 +650,10 @@ sub row2edit_1 {
 	    }
 	    $html .= "<input type=file name=$field>";
 	} elsif($type eq 'set' ) {
-	    $self->db()->dict_link($desc, $table, $field);
+	    $self->dict_link($desc, $table, $field);
 	    $html .= $self->choice($table, $desc, 'table', $field, $value, $use_default, 'checkbox');
 	} elsif($type eq 'enum') {
-	    $self->db()->dict_link($desc, $table, $field);
+	    $self->dict_link($desc, $table, $field);
 	    $html .= $self->choice($table, $desc, 'select', $field, $value, $use_default);
 	}
     }
@@ -746,7 +668,7 @@ sub imagedisplay {
 
     my($field) = $cgi->param('field');
     my($table) = $cgi->param('table');
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
     my($primary) = $cgi->param($info->{'_primary_'});
     my($image) = $self->exec_select_one("select $field from $table where $info->{'_primary_'} = $primary")->{$field};
 
@@ -770,14 +692,14 @@ sub row2title {
 sub row2view {
     my($self, $row, $table, $style) = @_;
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
     
     my($html);
     my($fields) = $info->{'_fields_'};
     my($field);
     foreach $field (@$fields) {
 	my($value) = $row->{$field};
-	my($quoted_value) = Ecila::tools::cgi::myescapeHTML($row->{$field});
+	my($quoted_value) = CGI::escapeHTML($row->{$field});
 	if($style eq 'short') {
 	    $html .= "<td>";
 	} else {
@@ -788,7 +710,7 @@ sub row2view {
 	if(defined($value)) {
 	    if($type eq 'char') {
 		if($style eq 'short' && length($value) > 30) {
-		    $quoted_value = Ecila::tools::cgi::myescapeHTML(substr($value, 0, 30)) . "...";
+		    $quoted_value = CGI::escapeHTML(substr($value, 0, 30)) . "...";
 		}
 	    } elsif($type eq 'blob' && defined($value)) {
 		my($imageutil) = $self->{'imageutil'};
@@ -951,7 +873,7 @@ sub dict_alt {
     dbg("dict_alt: check alt_value exists", "sqledit");
     return $param_value if(!defined($alt_value));
 
-    my($values) = $self->db()->dict_link($desc, $table, $field);
+    my($values) = $self->dict_link($desc, $table, $field);
 
     #
     # The value already exists, do nothing
@@ -962,7 +884,7 @@ sub dict_alt {
 
     if(!defined($result)) {
 	dbg("dict_alt: add", "sqledit");
-	$result = $self->db()->dict_add($desc->{'dict'}->{'table'}, $alt_value);
+	$result = $self->dict_add($desc->{'dict'}->{'table'}, $alt_value);
     }
     
     if(exists($desc->{'dict'}->{'map'})) {
@@ -991,7 +913,7 @@ sub row2assoc {
 sub row2assoc_1 {
     my($self, $table, $row, $assoc, $use_default) = @_;
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
     my($func) = sub {
 	my($field, $tag, $desc, $form) = @_;
@@ -1002,16 +924,16 @@ sub row2assoc_1 {
 #	warn("$field $tag $form $value " . ostring($desc));
 
 	if($type eq 'set' && $form eq 'CHECKBOX') {
-	    $self->db()->dict_link($desc, $table, $field);
+	    $self->dict_link($desc, $table, $field);
 	    $assoc->{$tag} = $self->choice($table, $desc, 'table', $field, $value, $use_default, 'checkbox');
 	} elsif($type eq 'set' && $form eq 'MENU') {
-	    $self->db()->dict_link($desc, $table, $field);
+	    $self->dict_link($desc, $table, $field);
 	    $assoc->{$tag} = $self->choice($table, $desc, 'select', $field, $value, $use_default, 'multiple');
 	} elsif($type eq 'enum' && $form eq 'RADIO') {
-	    $self->db()->dict_link($desc, $table, $field);
+	    $self->dict_link($desc, $table, $field);
 	    $assoc->{$tag} = $self->choice($table, $desc, 'table', $field, $value, $use_default, 'radio');
 	} elsif($type eq 'enum' && $form eq 'MENU') {
-	    $self->db()->dict_link($desc, $table, $field);
+	    $self->dict_link($desc, $table, $field);
 	    $assoc->{$tag} = $self->choice($table, $desc, 'select', $field, $value, $use_default);
 	} elsif($type eq 'blob' && defined($value)) {
 	    my($imageutil) = $self->{'imageutil'};
@@ -1020,7 +942,7 @@ sub row2assoc_1 {
 	} elsif(defined($value)) {
 	    $value = '' if(!defined($value));
 	    if($form eq 'QUOTED') {
-		$assoc->{$tag} = Ecila::tools::cgi::myescapeHTML($value);
+		$assoc->{$tag} = CGI::escapeHTML($value);
 	    } elsif($form eq 'CODED') {
 		$assoc->{$tag} = CGI::escape($value);
 	    } else {
@@ -1039,7 +961,7 @@ sub row2assoc_1 {
 sub walk_table_tags {
     my($self, $table, $assoc, $func) = @_;
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
     my(%sub);
     my($fields) = $info->{'_fields_'};
@@ -1087,9 +1009,9 @@ sub remove_confirm {
     $self->{'cgi'} = $cgi;
     my($table) = $cgi->param('table');
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
-    $self->db()->mdelete($table, "$info->{'_primary_'} = " . $cgi->param('primary'));
+    $self->mdelete($table, "$info->{'_primary_'} = " . $cgi->param('primary'));
 
     my($template) = $self->template("sqledit_remove_confirm");
 
@@ -1103,9 +1025,9 @@ sub edit {
     my($table) = $cgi->param('table');
     $self->{'cgi'} = $cgi;
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
     
-    my($row) = $self->db()->sexec_select_one($table, "select * from $table where $info->{'_primary_'} = " . $cgi->param('primary'));
+    my($row) = $self->sexec_select_one($table, "select * from $table where $info->{'_primary_'} = " . $cgi->param('primary'));
 
     my($template) = $self->template("sqledit_edit");
     my($assoc) = $template->{'assoc'};
@@ -1137,7 +1059,7 @@ sub args2row {
 
     my(%row);
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
     my($fields) = $info->{'_fields_'};
 
     my($field);
@@ -1183,9 +1105,9 @@ sub supdate_1 {
     my($self, $table, $primary) = @_;
     my($cgi) = $self->{'cgi'};
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
-    my($row) = $self->db()->sexec_select_one($table, "select * from $table where $info->{'_primary_'} = $primary");
+    my($row) = $self->sexec_select_one($table, "select * from $table where $info->{'_primary_'} = $primary");
 
     my(@updated_fields);
     my(%update);
@@ -1221,7 +1143,7 @@ sub supdate_1 {
 
     if(@updated_fields) {
 	$self->update_hook(\%update, $row, @updated_fields);
-	$self->db()->update($table, "$info->{'_primary_'} = $primary",
+	$self->update($table, "$info->{'_primary_'} = $primary",
 		      %update);
     }
     return \@updated_fields;
@@ -1244,7 +1166,7 @@ sub search_sql {
 	    my($type) = $info->{$field}->{'type'};
 	    $where .= "and $field ";
 	    if($type eq 'char') {
-		my($quoted_value) = $self->db()->quote($value);
+		my($quoted_value) = $self->quote($value);
 		$where .= "like '$quoted_value' ";
 	    } elsif($type eq 'int' || $type eq 'time') {
 		my($operator) = "=";
@@ -1292,7 +1214,7 @@ sub search {
     $self->{'cgi'} = $cgi;
     my($table) = $cgi->param('table');
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
     my($params) = $self->params();
 
@@ -1490,8 +1412,8 @@ sub hook_search {
 sub hook_search_retrieve {
     my($self, $table, $primary_value) = @_;
 
-    my($info) = $self->db()->info_table($table);
-    my($row) = $self->db()->sexec_select_one($table, "select * from $table where $info->{'_primary_'} = $primary_value");
+    my($info) = $self->info_table($table);
+    my($row) = $self->sexec_select_one($table, "select * from $table where $info->{'_primary_'} = $primary_value");
     return { $table => $row };
 }
 
@@ -1622,7 +1544,7 @@ sub searcher {
 sub searcher_sql {
     my($self, $template, $table, $context) = @_;
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
     return "select $info->{'_primary_'} from $table $context->{'real where'} $context->{'real order'}";
 }
@@ -1634,7 +1556,7 @@ sub searcher_pager {
     my($select) = $context->{'select'};
     if(!defined($select)) {
 	$select = sub {
-	    $self->db()->select(@_);
+	    $self->select(@_);
 	};
     }
 
@@ -1659,7 +1581,7 @@ sub searcher_nopager {
     my($select) = $context->{'select'};
     if(!defined($select)) {
 	$select = sub {
-	    $self->db()->select(@_);
+	    $self->select(@_);
 	};
     }
     
@@ -1776,7 +1698,7 @@ sub searcher_expand {
     my($self, $template, $tables, $rows, $context) = @_;
 
     my($table) = $tables->[0];
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
     my($primary_key) = $info->{'_primary_'};
     my($primary_values) = join(',', map { $_->{$primary_key} } @$rows);
     my($fields) = $self->searcher_select_fields($template, $table, $template->{'assoc'}, $context->{'fields'});
@@ -1795,7 +1717,7 @@ sub searcher_expand {
     error("no field to retrieve according to template entry") if(!defined($fields));
     my($sql) = "select $fields from $table where $primary_key in ($primary_values) $context->{'real order'}";
     dbg("searcher_layout: $sql", "sqledit");
-    ($rows) = $self->db()->sexec_select($table, $sql);
+    ($rows) = $self->sexec_select($table, $sql);
     my($results);
     my($row);
     foreach $row (@$rows) {
@@ -1851,7 +1773,7 @@ sub searcher_layout_result {
 sub searcher_links {
     my($self, $table, $row, $context) = @_;
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
     my($html);
     my($tag);
     foreach $tag ('edit', 'remove') {
@@ -1876,7 +1798,7 @@ sub searcher_select_fields {
     # Collect fields
     #
     my(@fields);
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
     my(@set_dict) = exists($info->{'_set_dict_'}) ? @{$info->{'_set_dict_'}} : ();
     if(exists($assoc->{'_DEFAULTROW_'})) {
 	dbg("table $table use _DEFAULTROW_", "sqledit");
@@ -1929,12 +1851,12 @@ sub search_1 {
     my($cgi) = $self->{'cgi'};
     my($table) = $cgi->param('table');
 
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
     #
     # Extract data
     #
-    my($rows, $rows_total) = $self->db()->sselect($table, $sql, $index, $length);
+    my($rows, $rows_total) = $self->sselect($table, $sql, $index, $length);
 
     my($links_set) = $self->links_set();
     my($links_set_root) = defined($links_set) ? $links_set->{$table} : undef;
@@ -1997,7 +1919,7 @@ sub list {
 
     my($html);
     my($file);
-    my($databases) = $self->db()->databases();
+    my($databases) = $self->databases();
     my($script) = $cgi->url(-absolute => 1);
     foreach $file (@$databases) {
 	my($base, $table) = $file =~ m;(.*)/(.*).frm;;
@@ -2019,7 +1941,7 @@ sub display_relations {
     my($self, $template, $table, $result, $level, $links_set, $index) = @_;
 
     my($block);
-    my($info) = $self->db()->info_table($table);
+    my($info) = $self->info_table($table);
 
     dbg("sqledit_search: display_relations for $table, level $level, relations for table " . join(",", keys(%$links_set)) . "\n", "sqledit");
 
@@ -2091,11 +2013,11 @@ sub extract_values {
 	my($linked_field) = $desc->{'field'};
 	my($field_to_link) = $desc->{'key'};
 	my($value_link) = $row->{$field_to_link};
-	my($info_link) = $self->db()->info_table($sub_table);
+	my($info_link) = $self->info_table($sub_table);
 	my($type_link) = $info_link->{$linked_field}->{'type'};
 	my($where_link) = "$linked_field ";
 	if($type_link eq 'char') {
-	    my($quoted_value) = $self->db()->quote($value_link);
+	    my($quoted_value) = $self->quote($value_link);
 	    $where_link .= "like '$quoted_value' ";
 	} elsif($type_link eq 'int' || $type_link eq 'time') {
 	    my($operator) = "=";
@@ -2115,7 +2037,7 @@ sub extract_values {
 	    $where_link .= "like '%$value_link%' ";
 	}
 	my($sql_link) = "select * from $sub_table where $where_link";
-	my($sub_rows, $sub_rows_total) = $self->db()->sexec_select($sub_table, $sql_link);
+	my($sub_rows, $sub_rows_total) = $self->sexec_select($sub_table, $sql_link);
 	@{$result->{$sub_table}} = map {
 	    {
 		'rows' => $_,
@@ -2323,6 +2245,22 @@ sub hidden {
     return $self->{'cgi'}->hidden($self->{'params'}, %pairs);
 }
 
+sub connect {
+    my($self) = @_;
+    my($connection);
+    eval {
+	$connection = $self->Ecila::tools::mysql::connect();
+    };
+    if($@) {
+	my($error) = $@;
+	my($cgi) = $self->{'cgi'};
+	my($url) = $cgi->url('-absolute' => 1);
+	$self->serror("Could not connect to database because: <p><pre>%s</pre> Try <a href=$url?context=confedit&file=mysql.conf>editing mysql.conf</a> and check that the database server is running.", $error);
+    } else {
+	return $connection;
+    }
+}
+
 sub confedit {
     my($self, $cgi) = @_;
     $self->{'cgi'} = $cgi;
@@ -2352,7 +2290,7 @@ sub confedit {
     my($cols) = $cgi->param('cols') || 80;
 
     my($template) = $self->template("edit");
-    template_set($template->{'assoc'}, '_TEXT_', Ecila::tools::cgi::myescapeHTML($text));
+    template_set($template->{'assoc'}, '_TEXT_', CGI::escapeHTML($text));
     template_set($template->{'assoc'}, '_COMMENT_', $comment);
     template_set($template->{'assoc'}, '_ROWS_', $rows);
     template_set($template->{'assoc'}, '_COLS_', $cols);
@@ -2363,13 +2301,7 @@ sub confedit {
 sub close {
     my($self) = @_;
 
-    $self->db()->logoff();
-}
-
-sub db {
-    my($self) = @_;
-
-    return $self->{'db'};
+    $self->logoff();
 }
 
 sub serror {
@@ -2377,7 +2309,6 @@ sub serror {
 
     my($template) = $self->template("error");
     my($message) = istring(@message);
-    $message = Carp::longmess($message) if $::opt_error_stack;
     template_set($template->{'assoc'}, '_MESSAGE_', $message);
     print $self->stemplate_build($template);
     error("HTMLIZED: $message");
